@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PNGImageDecoder.java 496559 2007-01-16 01:10:29Z cam $ */
+/* $Id: PNGImageDecoder.java 959327 2010-06-30 14:25:03Z spepping $ */
 
 package org.apache.xmlgraphics.image.codec.png;
 
@@ -39,12 +39,12 @@ import java.io.DataInputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.SequenceInputStream;
-import java.util.Date;
-import java.util.GregorianCalendar;
-import java.util.TimeZone;
-import java.util.List;
 import java.util.ArrayList;
 import java.util.Collections;
+import java.util.Date;
+import java.util.GregorianCalendar;
+import java.util.List;
+import java.util.TimeZone;
 import java.util.zip.Inflater;
 import java.util.zip.InflaterInputStream;
 
@@ -53,7 +53,7 @@ import org.apache.xmlgraphics.image.codec.util.PropertyUtil;
 import org.apache.xmlgraphics.image.codec.util.SimpleRenderedImage;
 
 /**
- * @version $Id: PNGImageDecoder.java 496559 2007-01-16 01:10:29Z cam $
+ * @version $Id: PNGImageDecoder.java 959327 2010-06-30 14:25:03Z spepping $
  */
 public class PNGImageDecoder extends ImageDecoderImpl {
 
@@ -393,8 +393,8 @@ class PNGImage extends SimpleRenderedImage {
                 String msg = PropertyUtil.getString("PNGImageDecoder0");
                 throw new RuntimeException(msg);
             }
-        } catch (Exception e) {
-            e.printStackTrace();
+        } catch (IOException ioe) {
+            ioe.printStackTrace();
             String msg = PropertyUtil.getString("PNGImageDecoder1");
             throw new RuntimeException(msg);
         }
