@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PropertyUtil.java 496556 2007-01-16 00:59:48Z cam $ */
+/* $Id: PropertyUtil.java 959327 2010-06-30 14:25:03Z spepping $ */
 
 package org.apache.xmlgraphics.image.codec.util;
 
@@ -25,7 +25,7 @@ import org.apache.xmlgraphics.util.i18n.LocalizableSupport;
 
 public class PropertyUtil {
     protected static final String RESOURCES =
-        "org.apache.xmlgraphics.image.codec.resources.Messages";
+        "org.apache.xmlgraphics.image.codec.Messages";
 
 
     protected static LocalizableSupport localizableSupport =
