@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,8 +15,8 @@
  * limitations under the License.
  */
 
-/* $Id: AbstractPSDocumentGraphics2D.java 592340 2007-11-06 09:04:04Z jeremias $ */
- 
+/* $Id: AbstractPSDocumentGraphics2D.java 959327 2010-06-30 14:25:03Z spepping $ */
+
 package org.apache.xmlgraphics.java2d.ps;
 
 import java.awt.Color;
@@ -27,6 +27,7 @@ import java.io.OutputStream;
 
 import org.apache.xmlgraphics.ps.DSCConstants;
 import org.apache.xmlgraphics.ps.PSGenerator;
+import org.apache.xmlgraphics.ps.PSProcSets;
 
 /**
  * This class is a wrapper for the <tt>PSGraphics2D</tt> that
@@ -34,16 +35,16 @@ import org.apache.xmlgraphics.ps.PSGenerator;
  * <tt>PSGraphics2D</tt>.
  *
  * @author <a href="mailto:keiron@aftexsw.com">Keiron Liddle</a>
- * @version $Id: AbstractPSDocumentGraphics2D.java 592340 2007-11-06 09:04:04Z jeremias $
+ * @version $Id: AbstractPSDocumentGraphics2D.java 959327 2010-06-30 14:25:03Z spepping $
  * @see org.apache.xmlgraphics.java2d.ps.PSGraphics2D
  */
 public abstract class AbstractPSDocumentGraphics2D extends PSGraphics2D {
 
     protected static final Integer ZERO = new Integer(0);
-    
+
     protected int width;
     protected int height;
-    
+
     protected float viewportWidth;
     protected float viewportHeight;
 
@@ -53,7 +54,7 @@ public abstract class AbstractPSDocumentGraphics2D extends PSGraphics2D {
     protected Shape initialClip;
     protected AffineTransform initialTransform;
 
-    
+
     /**
      * Create a new AbstractPSDocumentGraphics2D.
      * This is used to create a new PostScript document, the height,
@@ -85,10 +86,14 @@ public abstract class AbstractPSDocumentGraphics2D extends PSGraphics2D {
 
         //Setup for PostScript generation
         setPSGenerator(new PSGenerator(stream));
-        
+
         writeFileHeader();
     }
-    
+
+    /**
+     * Writes the file header.
+     * @throws IOException if an I/O error occurs
+     */
     protected abstract void writeFileHeader() throws IOException;
 
     /**
@@ -150,36 +155,57 @@ public abstract class AbstractPSDocumentGraphics2D extends PSGraphics2D {
         currentStream.write("Q\n");
         */
     }
-    
+
+    /**
+     * Returns the number of pages generated so far.
+     * @return the number of pages
+     */
     public int getPageCount() {
         return this.pagecount;
     }
 
+    /**
+     * Closes the current page and prepares to start a new one.
+     * @throws IOException if an I/O error occurs
+     */
     public void nextPage() throws IOException {
         closePage();
     }
 
+    /**
+     * Closes the current page.
+     * @throws IOException if an I/O error occurs
+     */
     protected void closePage() throws IOException {
         if (!this.pagePending) {
             return; //ignore
         }
         //Finish page
         writePageTrailer();
-        this.pagePending = false;         
+        this.pagePending = false;
     }
-    
+
     /**
      * Writes the page header for a page.
      * @throws IOException In case an I/O error occurs
      */
     protected abstract void writePageHeader() throws IOException;
-    
+
     /**
      * Writes the page trailer for a page.
      * @throws IOException In case an I/O error occurs
      */
     protected abstract void writePageTrailer() throws IOException;
-    
+
+    /**
+     * Writes the ProcSets ending up in the prolog to the PostScript file. Override to add your
+     * own ProcSets if so desired.
+     * @throws IOException In case an I/O error occurs
+     */
+    protected void writeProcSets() throws IOException {
+        PSProcSets.writeStdProcSet(gen);
+        PSProcSets.writeEPSProcSet(gen);
+    }
 
     /** {@inheritDoc} */
     public void preparePainting() {
@@ -193,33 +219,41 @@ public abstract class AbstractPSDocumentGraphics2D extends PSGraphics2D {
         }
     }
 
+    /**
+     * Starts a new page.
+     * @throws IOException if an I/O error occurs
+     */
     protected void startPage() throws IOException {
         if (this.pagePending) {
             throw new IllegalStateException("Close page first before starting another");
         }
         //Start page
         this.pagecount++;
-        
+
         if (this.initialTransform == null) {
             //Save initial transformation matrix
             this.initialTransform = getTransform();
-            this.initialClip = getClip();      
+            this.initialClip = getClip();
         } else {
             //Reset transformation matrix
             setTransform(this.initialTransform);
             setClip(this.initialClip);
         }
-          
+
         writePageHeader();
-        if ((this.viewportWidth != this.width 
+        AffineTransform at;
+        if ((this.viewportWidth != this.width
                 || this.viewportHeight != this.height)
                 && (this.viewportWidth > 0) && (this.viewportHeight > 0)){
-            gen.concatMatrix(this.width / this.viewportWidth, 0, 
-                       0, -1 * (this.height / this.viewportHeight), 
+            at = new AffineTransform(this.width / this.viewportWidth, 0,
+                       0, -1 * (this.height / this.viewportHeight),
                        0, this.height);
         } else {
-            gen.concatMatrix(1, 0, 0, -1, 0, this.height);
+            at = new AffineTransform(1, 0, 0, -1, 0, this.height);
         }
+        // Do not use concatMatrix, since it alters PSGenerator current state
+        //gen.concatMatrix(at);
+        gen.writeln(gen.formatMatrix(at) + " " + gen.mapCommand("concat"));
         gen.writeDSCComment(DSCConstants.END_PAGE_SETUP);
         this.pagePending = true;
     }
@@ -236,14 +270,14 @@ public abstract class AbstractPSDocumentGraphics2D extends PSGraphics2D {
         if (this.pagePending) {
             closePage();
         }
-        
+
         //Finish document
         gen.writeDSCComment(DSCConstants.TRAILER);
         gen.writeDSCComment(DSCConstants.PAGES, new Integer(this.pagecount));
         gen.writeDSCComment(DSCConstants.EOF);
         gen.flush();
     }
-    
+
     /**
      * This constructor supports the create method
      * @param g the PostScript document graphics to make a copy of
