@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,20 +15,19 @@
  * limitations under the License.
  */
 
-/* $Id: EPSDocumentGraphics2D.java 603586 2007-12-12 12:13:57Z acumiskey $ */
- 
+/* $Id: EPSDocumentGraphics2D.java 959327 2010-06-30 14:25:03Z spepping $ */
+
 package org.apache.xmlgraphics.java2d.ps;
 
 import java.io.IOException;
 
 import org.apache.xmlgraphics.ps.DSCConstants;
-import org.apache.xmlgraphics.ps.PSProcSets;
 
 /**
  * This class is a wrapper for the <tt>AbstractPSDocumentGraphics2D</tt> that
  * is used to create EPS (Encapsulated PostScript) files instead of PS file.
  *
- * @version $Id: EPSDocumentGraphics2D.java 603586 2007-12-12 12:13:57Z acumiskey $
+ * @version $Id: EPSDocumentGraphics2D.java 959327 2010-06-30 14:25:03Z spepping $
  * @see org.apache.xmlgraphics.java2d.ps.PSGraphics2D
  * @see org.apache.xmlgraphics.java2d.ps.AbstractPSDocumentGraphics2D
  */
@@ -49,36 +48,37 @@ public class EPSDocumentGraphics2D extends AbstractPSDocumentGraphics2D {
         super(textAsShapes);
     }
 
+    /** {@inheritDoc} */
     protected void writeFileHeader() throws IOException {
         final Long pagewidth = new Long(this.width);
         final Long pageheight = new Long(this.height);
 
         //PostScript Header
         gen.writeln(DSCConstants.PS_ADOBE_30 + " " + DSCConstants.EPSF_30);
-        gen.writeDSCComment(DSCConstants.CREATOR, 
-                    new String[] {"Apache XML Graphics Commons" 
+        gen.writeDSCComment(DSCConstants.CREATOR,
+                    new String[] {"Apache XML Graphics Commons"
                         + ": EPS Generator for Java2D"});
-        gen.writeDSCComment(DSCConstants.CREATION_DATE, 
+        gen.writeDSCComment(DSCConstants.CREATION_DATE,
                     new Object[] {new java.util.Date()});
         gen.writeDSCComment(DSCConstants.PAGES, DSCConstants.ATEND);
         gen.writeDSCComment(DSCConstants.BBOX, new Object[]
                 {ZERO, ZERO, pagewidth, pageheight});
         gen.writeDSCComment(DSCConstants.LANGUAGE_LEVEL, new Integer(gen.getPSLevel()));
         gen.writeDSCComment(DSCConstants.END_COMMENTS);
-        
+
         //Prolog
         gen.writeDSCComment(DSCConstants.BEGIN_PROLOG);
-        PSProcSets.writeStdProcSet(gen);
-        PSProcSets.writeEPSProcSet(gen);
+        writeProcSets();
         if (customTextHandler instanceof PSTextHandler) {
             ((PSTextHandler)customTextHandler).writeSetup();
         }
         gen.writeDSCComment(DSCConstants.END_PROLOG);
     }
 
+    /** {@inheritDoc} */
     protected void writePageHeader() throws IOException {
         Integer pageNumber = new Integer(this.pagecount);
-        gen.writeDSCComment(DSCConstants.PAGE, new Object[] 
+        gen.writeDSCComment(DSCConstants.PAGE, new Object[]
                 {pageNumber.toString(), pageNumber});
         gen.writeDSCComment(DSCConstants.PAGE_BBOX, new Object[]
                 {ZERO, ZERO, new Integer(width), new Integer(height)});
@@ -87,7 +87,8 @@ public class EPSDocumentGraphics2D extends AbstractPSDocumentGraphics2D {
             ((PSTextHandler)customTextHandler).writePageSetup();
         }
     }
-    
+
+    /** {@inheritDoc} */
     protected void writePageTrailer() throws IOException {
         //nop, no DSC PageTrailer needed
     }
