@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PSDocumentGraphics2D.java 603586 2007-12-12 12:13:57Z acumiskey $ */
+/* $Id: PSDocumentGraphics2D.java 959327 2010-06-30 14:25:03Z spepping $ */
 
 package org.apache.xmlgraphics.java2d.ps;
 
@@ -23,7 +23,6 @@ import java.io.IOException;
 import java.io.OutputStream;
 
 import org.apache.xmlgraphics.ps.DSCConstants;
-import org.apache.xmlgraphics.ps.PSProcSets;
 
 /**
  * This class is a wrapper for the <tt>PSGraphics2D</tt> that
@@ -31,7 +30,7 @@ import org.apache.xmlgraphics.ps.PSProcSets;
  * <tt>PSGraphics2D</tt>.
  *
  * @author <a href="mailto:keiron@aftexsw.com">Keiron Liddle</a>
- * @version $Id: PSDocumentGraphics2D.java 603586 2007-12-12 12:13:57Z acumiskey $
+ * @version $Id: PSDocumentGraphics2D.java 959327 2010-06-30 14:25:03Z spepping $
  * @see org.apache.xmlgraphics.java2d.ps.PSGraphics2D
  */
 public class PSDocumentGraphics2D extends AbstractPSDocumentGraphics2D {
@@ -71,10 +70,12 @@ public class PSDocumentGraphics2D extends AbstractPSDocumentGraphics2D {
         setupDocument(stream, width, height);
     }
 
+    /** {@inheritDoc} */
     public void nextPage() throws IOException {
         closePage();
     }
 
+    /** {@inheritDoc} */
     protected void writeFileHeader() throws IOException {
         final Long pagewidth = new Long(this.width);
         final Long pageheight = new Long(this.height);
@@ -82,7 +83,7 @@ public class PSDocumentGraphics2D extends AbstractPSDocumentGraphics2D {
         //PostScript Header
         gen.writeln(DSCConstants.PS_ADOBE_30);
         gen.writeDSCComment(DSCConstants.CREATOR,
-                    new String[] {"Apache XML Graphics Commons" 
+                    new String[] {"Apache XML Graphics Commons"
                         + ": PostScript Generator for Java2D"});
         gen.writeDSCComment(DSCConstants.CREATION_DATE,
                     new Object[] {new java.util.Date()});
@@ -91,28 +92,28 @@ public class PSDocumentGraphics2D extends AbstractPSDocumentGraphics2D {
                 {ZERO, ZERO, pagewidth, pageheight});
         gen.writeDSCComment(DSCConstants.LANGUAGE_LEVEL, new Integer(gen.getPSLevel()));
         gen.writeDSCComment(DSCConstants.END_COMMENTS);
-        
+
         //Defaults
         gen.writeDSCComment(DSCConstants.BEGIN_DEFAULTS);
         gen.writeDSCComment(DSCConstants.END_DEFAULTS);
-        
+
         //Prolog
         gen.writeDSCComment(DSCConstants.BEGIN_PROLOG);
         gen.writeDSCComment(DSCConstants.END_PROLOG);
-        
+
         //Setup
         gen.writeDSCComment(DSCConstants.BEGIN_SETUP);
-        PSProcSets.writeStdProcSet(gen);
-        PSProcSets.writeEPSProcSet(gen);
+        writeProcSets();
         if (customTextHandler instanceof PSTextHandler) {
             ((PSTextHandler)customTextHandler).writeSetup();
         }
         gen.writeDSCComment(DSCConstants.END_SETUP);
     }
 
+    /** {@inheritDoc} */
     protected void writePageHeader() throws IOException {
         Integer pageNumber = new Integer(this.pagecount);
-        gen.writeDSCComment(DSCConstants.PAGE, new Object[] 
+        gen.writeDSCComment(DSCConstants.PAGE, new Object[]
                 {pageNumber.toString(), pageNumber});
         gen.writeDSCComment(DSCConstants.PAGE_BBOX, new Object[]
                 {ZERO, ZERO, new Integer(width), new Integer(height)});
@@ -125,11 +126,12 @@ public class PSDocumentGraphics2D extends AbstractPSDocumentGraphics2D {
             ((PSTextHandler)customTextHandler).writePageSetup();
         }
     }
-    
+
+    /** {@inheritDoc} */
     protected void writePageTrailer() throws IOException {
-        gen.writeln("showpage");        
+        gen.showPage();
     }
-    
+
     /**
      * This constructor supports the create method
      * @param g the PostScript document graphics to make a copy of
