@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PSGraphics2D.java 603586 2007-12-12 12:13:57Z acumiskey $ */
+/* $Id: PSGraphics2D.java 949528 2010-05-30 15:18:51Z jeremias $ */
 
 package org.apache.xmlgraphics.java2d.ps;
 
@@ -24,8 +24,6 @@ import java.awt.AlphaComposite;
 import java.awt.BasicStroke;
 import java.awt.Color;
 import java.awt.Dimension;
-/* java.awt.Font is not imported to avoid confusion with 
-      other classes called "Font" */ 
 import java.awt.GradientPaint;
 import java.awt.Graphics;
 import java.awt.Graphics2D;
@@ -38,7 +36,9 @@ import java.awt.Shape;
 import java.awt.Stroke;
 import java.awt.TexturePaint;
 import java.awt.geom.AffineTransform;
+import java.awt.geom.Area;
 import java.awt.geom.PathIterator;
+import java.awt.geom.Rectangle2D;
 import java.awt.image.BufferedImage;
 import java.awt.image.ImageObserver;
 import java.awt.image.RenderedImage;
@@ -58,28 +58,30 @@ import org.apache.xmlgraphics.ps.PSImageUtils;
  * code.
  *
  * @author <a href="mailto:keiron@aftexsw.com">Keiron Liddle</a>
- * @version $Id: PSGraphics2D.java 603586 2007-12-12 12:13:57Z acumiskey $
+ * @version $Id: PSGraphics2D.java 949528 2010-05-30 15:18:51Z jeremias $
  * @see org.apache.xmlgraphics.java2d.AbstractGraphics2D
  */
 public class PSGraphics2D extends AbstractGraphics2D {
 
     private static final AffineTransform IDENTITY_TRANSFORM = new AffineTransform();
-    
+
     private static final boolean DEBUG = false;
-    
+
     protected PSGraphics2D rootG2D;
-    
+
     /** the PostScript generator being created */
     protected PSGenerator gen;
 
-    private boolean clippingDisabled = false;
+    /** Disable or enable clipping */
+    protected boolean clippingDisabled = false;
 
     /** Fallback text handler */
-    protected TextHandler fallbackTextHandler = new StrokingTextHandler(this);
-    
+
+    protected TextHandler fallbackTextHandler = new StrokingTextHandler();
+
     /** Custom text handler */
     protected TextHandler customTextHandler;
-    
+
     /**
      * the current colour for use in svg
      */
@@ -115,7 +117,10 @@ public class PSGraphics2D extends AbstractGraphics2D {
         this.rootG2D = (g.rootG2D != null ? g.rootG2D : g);
         setPSGenerator(g.gen);
         this.clippingDisabled = g.clippingDisabled;
-        this.fallbackTextHandler = g.fallbackTextHandler;
+        //this.fallbackTextHandler is not copied
+        //TODO The customTextHandler should probably not be passed over just like that
+        //fallbackTextHandler, for example, has to be recreated to point to the sub-Graphics2D
+        //to get the text positioning right. This might require changes in the TextHandler interface
         this.customTextHandler = g.customTextHandler;
         this.currentColour    = g.currentColour;
     }
@@ -161,13 +166,22 @@ public class PSGraphics2D extends AbstractGraphics2D {
     public void setCustomTextHandler(TextHandler handler) {
         this.customTextHandler = handler;
     }
-    
+
     /* TODO Add me back at the right place!!!
     private void setPrivateHints() {
-        setRenderingHint(RenderingHintsKeyExt.KEY_AVOID_TILE_PAINTING, 
+        setRenderingHint(RenderingHintsKeyExt.KEY_AVOID_TILE_PAINTING,
                 RenderingHintsKeyExt.VALUE_AVOID_TILE_PAINTING_ON);
     }*/
 
+    /**
+     * Disable clipping on each draw command.
+     *
+     * @param b set to true to disable all clipping.
+     */
+    public void disableClipping(boolean b) {
+        this.clippingDisabled = b;
+    }
+
     /**
      * Creates a new <code>Graphics</code> object that is
      * a copy of this <code>Graphics</code> object.
@@ -258,7 +272,7 @@ public class PSGraphics2D extends AbstractGraphics2D {
             gen.concatMatrix(at);
             Shape imclip = getClip();
             writeClip(imclip);
-            PSImageUtils.renderBitmapImage(buf, 
+            PSImageUtils.renderBitmapImage(buf,
                 x, y, width, height, gen);
             gen.restoreGraphicsState();
         } catch (IOException ioe) {
@@ -354,7 +368,26 @@ public class PSGraphics2D extends AbstractGraphics2D {
     }
 
     /**
-     * Processes a path iterator generating the nexessary painting operations.
+     * Processes a Shape generating the necessary painting operations.
+     * @param s Shape to process
+     * @return the winding rule of the path defining the shape
+     * @throws IOException In case of an I/O problem.
+     */
+    public int processShape(Shape s) throws IOException {
+        if (s instanceof Rectangle2D) {
+            // Special optimization in case of Rectangle Shape
+            Rectangle2D r = (Rectangle2D) s;
+            gen.defineRect(r.getX(), r.getY(), r.getWidth(), r.getHeight());
+            return PathIterator.WIND_NON_ZERO;
+        } else {
+            PathIterator iter = s.getPathIterator(IDENTITY_TRANSFORM);
+            processPathIterator(iter);
+            return iter.getWindingRule();
+        }
+    }
+
+    /**
+     * Processes a path iterator generating the necessary painting operations.
      * @param iter PathIterator to process
      * @throws IOException In case of an I/O problem.
      */
@@ -369,27 +402,27 @@ public class PSGraphics2D extends AbstractGraphics2D {
                                  + gen.formatDouble(vals[2]) + " "
                                  + gen.formatDouble(vals[3]) + " "
                                  + gen.formatDouble(vals[4]) + " "
-                                 + gen.formatDouble(vals[5])
-                                 + " curveto");
+                                 + gen.formatDouble(vals[5]) + " "
+                                 + gen.mapCommand("curveto"));
                 break;
             case PathIterator.SEG_LINETO:
                 gen.writeln(gen.formatDouble(vals[0]) + " "
-                                 + gen.formatDouble(vals[1])
-                                 + " lineto");
+                                 + gen.formatDouble(vals[1]) + " "
+                                 + gen.mapCommand("lineto"));
                 break;
             case PathIterator.SEG_MOVETO:
                 gen.writeln(gen.formatDouble(vals[0]) + " "
-                                 + gen.formatDouble(vals[1])
-                                 + " M");
+                                 + gen.formatDouble(vals[1]) + " "
+                                 + gen.mapCommand("moveto"));
                 break;
             case PathIterator.SEG_QUADTO:
-                gen.writeln(gen.formatDouble(vals[0]) + " " 
-                          + gen.formatDouble(vals[1]) + " " 
-                          + gen.formatDouble(vals[2]) + " " 
-                          + gen.formatDouble(vals[3]) + " QUADTO ");
+                gen.writeln(gen.formatDouble(vals[0]) + " "
+                          + gen.formatDouble(vals[1]) + " "
+                          + gen.formatDouble(vals[2]) + " "
+                          + gen.formatDouble(vals[3]) + " QT");
                 break;
             case PathIterator.SEG_CLOSE:
-                gen.writeln("closepath");
+                gen.writeln(gen.mapCommand("closepath"));
                 break;
             default:
                 break;
@@ -418,7 +451,7 @@ public class PSGraphics2D extends AbstractGraphics2D {
         preparePainting();
         try {
             gen.saveGraphicsState();
-            
+
             AffineTransform trans = getTransform();
             boolean newTransform = gen.getCurrentState().checkTransform(trans)
                     && !trans.isIdentity();
@@ -427,15 +460,16 @@ public class PSGraphics2D extends AbstractGraphics2D {
                 gen.concatMatrix(trans);
             }
             Shape imclip = getClip();
-            writeClip(imclip);
+            if (shouldBeClipped(imclip, s)) {
+                writeClip(imclip);
+            }
             establishColor(getColor());
 
             applyPaint(getPaint(), false);
             applyStroke(getStroke());
 
-            gen.writeln("newpath");
-            PathIterator iter = s.getPathIterator(IDENTITY_TRANSFORM);
-            processPathIterator(iter);
+            gen.writeln(gen.mapCommand("newpath"));
+            processShape(s);
             doDrawing(false, true, false);
             gen.restoreGraphicsState();
         } catch (IOException ioe) {
@@ -443,6 +477,22 @@ public class PSGraphics2D extends AbstractGraphics2D {
         }
     }
 
+    /**
+     * Determines if a shape interacts with a clipping region.
+     * @param clip Shape defining the clipping region
+     * @param s Shape to be drawn
+     * @return true if code for a clipping region needs to be generated.
+     */
+    public boolean shouldBeClipped(Shape clip, Shape s) {
+        if (clip == null || s == null) {
+            return false;
+        }
+        Area as = new Area(s);
+        Area imclip = new Area(clip);
+        imclip.intersect(as);
+        return !imclip.equals(as);
+    }
+
     /**
      * Establishes a clipping region
      * @param s Shape defining the clipping region
@@ -454,11 +504,10 @@ public class PSGraphics2D extends AbstractGraphics2D {
         if (!this.clippingDisabled) {
             preparePainting();
             try {
-                gen.writeln("newpath");
-                PathIterator iter = s.getPathIterator(IDENTITY_TRANSFORM);
-                processPathIterator(iter);
+                gen.writeln(gen.mapCommand("newpath"));
+                processShape(s);
                 // clip area
-                gen.writeln("clip");
+                gen.writeln(gen.mapCommand("clip"));
             } catch (IOException ioe) {
                 handleIOException(ioe);
             }
@@ -475,7 +524,19 @@ public class PSGraphics2D extends AbstractGraphics2D {
         if (paint instanceof GradientPaint) {
             System.err.println("NYI: Gradient paint");
         } else if (paint instanceof TexturePaint) {
-            System.err.println("NYI: texture paint");
+            if (fill) {
+                try {
+                    // create pattern with texture and use it for filling of a graphics object
+                    PSTilingPattern psTilingPattern = new PSTilingPattern("Pattern1",
+                            (TexturePaint)paint, 0, 0, 3, null);
+                    gen.write(psTilingPattern.toString());
+                    gen.writeln("/Pattern " + gen.mapCommand("setcolorspace"));
+                    gen.writeln(psTilingPattern.getName() + " " + gen.mapCommand("setcolor"));
+                }
+                catch (IOException ioe) {
+                    handleIOException(ioe);
+                }
+            }
         }
     }
 
@@ -486,59 +547,75 @@ public class PSGraphics2D extends AbstractGraphics2D {
     protected void applyStroke(Stroke stroke) {
         preparePainting();
         try {
-            if (stroke instanceof BasicStroke) {
-                BasicStroke bs = (BasicStroke)stroke;
-
-                float[] da = bs.getDashArray();
-                if (da != null) {
-                    gen.write("[");
-                    for (int count = 0; count < da.length; count++) {
-                        gen.write(gen.formatDouble(da[count]));
-                        if (count < da.length - 1) {
-                            gen.write(" ");
-                        }
-                    }
-                    gen.write("] ");
-                    float offset = bs.getDashPhase();
-                    gen.writeln(gen.formatDouble(offset) + " setdash");
-                }
-                int ec = bs.getEndCap();
-                switch (ec) {
-                case BasicStroke.CAP_BUTT:
-                    gen.writeln("0 setlinecap");
-                    break;
-                case BasicStroke.CAP_ROUND:
-                    gen.writeln("1 setlinecap");
-                    break;
-                case BasicStroke.CAP_SQUARE:
-                    gen.writeln("2 setlinecap");
-                    break;
-                default: System.err.println("Unsupported line cap: " + ec);
-                }
-
-                int lj = bs.getLineJoin();
-                switch (lj) {
-                case BasicStroke.JOIN_MITER:
-                    gen.writeln("0 setlinejoin");
-                    float ml = bs.getMiterLimit();
-                    gen.writeln(gen.formatDouble(ml >= -1 ? ml : 1) + " setmiterlimit");
-                    break;
-                case BasicStroke.JOIN_ROUND:
-                    gen.writeln("1 setlinejoin");
-                    break;
-                case BasicStroke.JOIN_BEVEL:
-                    gen.writeln("2 setlinejoin");
-                    break;
-                default: System.err.println("Unsupported line join: " + lj);
-                }
-                float lw = bs.getLineWidth();
-                gen.writeln(gen.formatDouble(lw) + " setlinewidth");
-            }
+            applyStroke(stroke, gen);
         } catch (IOException ioe) {
             handleIOException(ioe);
         }
     }
 
+    /**
+     * Applies a new Stroke object.
+     * @param stroke the Stroke instance
+     * @param gen the PS generator
+     * @throws IOException if an I/O error occurs
+     */
+    public static void applyStroke(Stroke stroke, final PSGenerator gen)
+            throws IOException {
+        if (stroke instanceof BasicStroke) {
+            BasicStroke basicStroke = (BasicStroke)stroke;
+
+            float[] da = basicStroke.getDashArray();
+            if (da != null) {
+                StringBuffer sb = new StringBuffer("[");
+                for (int count = 0; count < da.length; count++) {
+                    sb.append(gen.formatDouble(da[count]));
+                    if (count < da.length - 1) {
+                        sb.append(" ");
+                    }
+                }
+                sb.append("] ");
+                float offset = basicStroke.getDashPhase();
+                sb.append(gen.formatDouble(offset));
+                gen.useDash(sb.toString());
+            } else {
+                gen.useDash(null);
+            }
+            int ec = basicStroke.getEndCap();
+            switch (ec) {
+            case BasicStroke.CAP_BUTT:
+                gen.useLineCap(0);
+                break;
+            case BasicStroke.CAP_ROUND:
+                gen.useLineCap(1);
+                break;
+            case BasicStroke.CAP_SQUARE:
+                gen.useLineCap(2);
+                break;
+            default: System.err.println("Unsupported line cap: " + ec);
+            }
+
+            int lj = basicStroke.getLineJoin();
+            switch (lj) {
+            case BasicStroke.JOIN_MITER:
+                gen.useLineJoin(0);
+                float ml = basicStroke.getMiterLimit();
+                gen.useMiterLimit(ml >= -1 ? ml : 1);
+                break;
+            case BasicStroke.JOIN_ROUND:
+                gen.useLineJoin(1);
+                break;
+            case BasicStroke.JOIN_BEVEL:
+                gen.useLineJoin(2);
+                break;
+            default: System.err.println("Unsupported line join: " + lj);
+            }
+            float lw = basicStroke.getLineWidth();
+            gen.useLineWidth(lw);
+        } else {
+            System.err.println("Stroke not supported: " + stroke.toString());
+        }
+    }
+
     /**
      * Renders a {@link RenderedImage},
      * applying a transform from image
@@ -561,7 +638,19 @@ public class PSGraphics2D extends AbstractGraphics2D {
      */
     public void drawRenderedImage(RenderedImage img, AffineTransform xform) {
         preparePainting();
-        System.err.println("NYI: drawRenderedImage");
+        try {
+            AffineTransform at = getTransform();
+            gen.saveGraphicsState();
+            gen.concatMatrix(at);
+            gen.concatMatrix(xform);
+            Shape imclip = getClip();
+            writeClip(imclip);
+            PSImageUtils.renderBitmapImage(img,
+                0, 0, img.getWidth(), img.getHeight(), gen);
+            gen.restoreGraphicsState();
+        } catch (IOException ioe) {
+            handleIOException(ioe);
+        }
     }
 
     /**
@@ -636,9 +725,9 @@ public class PSGraphics2D extends AbstractGraphics2D {
     public void drawString(String s, float x, float y) {
         try {
             if (customTextHandler != null && !textAsShapes) {
-                customTextHandler.drawString(s, x, y);
+                customTextHandler.drawString(this, s, x, y);
             } else {
-                fallbackTextHandler.drawString(s, x, y);
+                fallbackTextHandler.drawString(this, s, x, y);
             }
         } catch (IOException ioe) {
             handleIOException(ioe);
@@ -663,26 +752,27 @@ public class PSGraphics2D extends AbstractGraphics2D {
         preparePainting();
         try {
             gen.saveGraphicsState();
-            
+
             AffineTransform trans = getTransform();
             boolean newTransform = gen.getCurrentState().checkTransform(trans)
                     && !trans.isIdentity();
-            
+
             if (newTransform) {
                 gen.concatMatrix(trans);
             }
             Shape imclip = getClip();
-            writeClip(imclip);
-            
+            if (shouldBeClipped(imclip, s)) {
+                writeClip(imclip);
+            }
+
             establishColor(getColor());
 
             applyPaint(getPaint(), true);
 
-            gen.writeln("newpath");
-            PathIterator iter = s.getPathIterator(IDENTITY_TRANSFORM);
-            processPathIterator(iter);
+            gen.writeln(gen.mapCommand("newpath"));
+            int windingRule = processShape(s);
             doDrawing(true, false,
-                      iter.getWindingRule() == PathIterator.WIND_EVEN_ODD);
+                    windingRule == PathIterator.WIND_EVEN_ODD);
             gen.restoreGraphicsState();
         } catch (IOException ioe) {
             handleIOException(ioe);
@@ -696,26 +786,32 @@ public class PSGraphics2D extends AbstractGraphics2D {
      * @param nonzero true if the non-zero winding rule should be used when filling
      * @exception IOException In case of an I/O problem
      */
-    protected void doDrawing(boolean fill, boolean stroke, boolean nonzero) 
+    protected void doDrawing(boolean fill, boolean stroke, boolean nonzero)
                 throws IOException {
         preparePainting();
         if (fill) {
             if (stroke) {
                 if (!nonzero) {
-                    gen.writeln("gsave fill grestore stroke");
+                    gen.writeln(gen.mapCommand("gsave") + " "
+                            + gen.mapCommand("fill") + " "
+                            + gen.mapCommand("grestore") + " "
+                            + gen.mapCommand("stroke"));
                 } else {
-                    gen.writeln("gsave eofill grestore stroke");
+                    gen.writeln(gen.mapCommand("gsave") + " "
+                            + gen.mapCommand("eofill") + " "
+                            + gen.mapCommand("grestore") + " "
+                            + gen.mapCommand("stroke"));
                 }
             } else {
                 if (!nonzero) {
-                    gen.writeln("fill");
+                    gen.writeln(gen.mapCommand("fill"));
                 } else {
-                    gen.writeln("eofill");
+                    gen.writeln(gen.mapCommand("eofill"));
                 }
             }
         } else {
             // if(stroke)
-            gen.writeln("stroke");
+            gen.writeln(gen.mapCommand("stroke"));
         }
     }
 
