@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,49 +15,92 @@
  * limitations under the License.
  */
 
-/* $Id: PSProcSets.java 568176 2007-08-21 15:58:15Z cbowditch $ */
- 
+/* $Id: PSProcSets.java 924840 2010-03-18 14:52:36Z jeremias $ */
+
 package org.apache.xmlgraphics.ps;
 
 import java.io.IOException;
+import java.util.Collections;
+import java.util.Iterator;
+import java.util.Map;
 
 /**
  * This class defines the basic resources (procsets) used by the Apache XML Graphics project.
- * 
- * @version $Id: PSProcSets.java 568176 2007-08-21 15:58:15Z cbowditch $
+ *
+ * @version $Id: PSProcSets.java 924840 2010-03-18 14:52:36Z jeremias $
  */
 public final class PSProcSets {
 
     /** the standard procset for the XML Graphics project */
-    public static final PSResource STD_PROCSET = new StdProcSet();
+    public static final PSResource STD_PROCSET;
     /** the EPS procset for the XML Graphics project */
     public static final PSResource EPS_PROCSET = new EPSProcSet();
-    
-    private static class StdProcSet extends PSProcSet {
-        
-        public StdProcSet() {
-            super("Apache XML Graphics Std ProcSet", 1.0f, 0);
+
+    /** the standard command map matching the {@link #STD_PROCSET}. */
+    public static final PSCommandMap STD_COMMAND_MAP;
+
+    static {
+        StdProcSet stdProcSet = new StdProcSet();
+        STD_PROCSET = stdProcSet;
+        STD_COMMAND_MAP = stdProcSet;
+    }
+
+    /**
+     * The standard procset used by XML Graphics Commons.
+     */
+    private static class StdProcSet extends PSProcSet implements PSCommandMap {
+
+        /** A Map<String, String> of standard shorthand macros defined in the {@link StdProcSet}. */
+        private static final Map STANDARD_MACROS;
+
+        static {
+            Map macros = new java.util.HashMap();
+            macros.put("moveto", "M");
+            macros.put("rmoveto", "RM");
+            macros.put("curveto", "C");
+            macros.put("lineto", "L");
+            macros.put("show", "t");
+            macros.put("ashow", "A");
+            macros.put("closepath", "cp");
+            macros.put("setrgbcolor", "RC");
+            macros.put("setgray", "GC");
+            macros.put("setcmykcolor", "CC");
+            macros.put("newpath", "N");
+            macros.put("setmiterlimit", "ML");
+            macros.put("setlinewidth", "LC");
+            macros.put("setlinewidth", "LW");
+            macros.put("setlinejoin", "LJ");
+            macros.put("grestore", "GR");
+            macros.put("gsave", "GS");
+            macros.put("fill", "f");
+            macros.put("stroke", "S");
+            macros.put("concat", "CT");
+            STANDARD_MACROS = Collections.unmodifiableMap(macros);
         }
-        
+
+        public StdProcSet() {
+            super("Apache XML Graphics Std ProcSet", 1.1f, 0);
+        }
+
         public void writeTo(PSGenerator gen) throws IOException {
-            gen.writeDSCComment(DSCConstants.BEGIN_RESOURCE, 
-                    new Object[] {TYPE_PROCSET, getName(), 
+            gen.writeDSCComment(DSCConstants.BEGIN_RESOURCE,
+                    new Object[] {TYPE_PROCSET, getName(),
                         Float.toString(getVersion()), Integer.toString(getRevision())});
-            gen.writeDSCComment(DSCConstants.VERSION, 
+            gen.writeDSCComment(DSCConstants.VERSION,
                     new Object[] {Float.toString(getVersion()), Integer.toString(getRevision())});
-            gen.writeDSCComment(DSCConstants.COPYRIGHT, "Copyright 2001-2003 "
+            gen.writeDSCComment(DSCConstants.COPYRIGHT, "Copyright 2001-2003,2010 "
                         + "The Apache Software Foundation. "
                         + "License terms: http://www.apache.org/licenses/LICENSE-2.0");
-            gen.writeDSCComment(DSCConstants.TITLE, 
+            gen.writeDSCComment(DSCConstants.TITLE,
                     "Basic set of procedures used by the XML Graphics project (Batik and FOP)");
 
             gen.writeln("/bd{bind def}bind def");
             gen.writeln("/ld{load def}bd");
-            gen.writeln("/M/moveto ld");
-            gen.writeln("/RM/rmoveto ld");
-            gen.writeln("/t/show ld");
-            gen.writeln("/A/ashow ld");
-            gen.writeln("/cp/closepath ld");
+            Iterator iter = STANDARD_MACROS.entrySet().iterator();
+            while (iter.hasNext()) {
+                Map.Entry entry = (Map.Entry)iter.next();
+                gen.writeln("/" + entry.getValue() + "/" + entry.getKey()+" ld");
+            }
 
             gen.writeln("/re {4 2 roll M"); //define rectangle
             gen.writeln("1 index 0 rlineto");
@@ -68,12 +111,12 @@ public final class PSProcSets {
             gen.writeln("/_ctm matrix def"); //Holds the current matrix
             gen.writeln("/_tm matrix def");
             //BT: save currentmatrix, set _tm to identitymatrix and move to 0/0
-            gen.writeln("/BT { _ctm currentmatrix pop matrix _tm copy pop 0 0 moveto } bd"); 
+            gen.writeln("/BT { _ctm currentmatrix pop matrix _tm copy pop 0 0 moveto } bd");
             //ET: restore last currentmatrix
             gen.writeln("/ET { _ctm setmatrix } bd");
             gen.writeln("/iTm { _ctm setmatrix _tm concat } bd");
             gen.writeln("/Tm { _tm astore pop iTm 0 0 moveto } bd");
-            
+
             gen.writeln("/ux 0.0 def");
             gen.writeln("/uy 0.0 def");
 
@@ -126,8 +169,8 @@ public final class PSProcSets {
             gen.writeln("  Tt setlinewidth stroke");
             gen.writeln("  grestore");
             gen.writeln("} bd");
-            
-            gen.writeln("/QUADTO {");
+
+            gen.writeln("/QT {");
             gen.writeln("/Y22 exch store");
             gen.writeln("/X22 exch store");
             gen.writeln("/Y21 exch store");
@@ -139,7 +182,7 @@ public final class PSProcSets {
             gen.writeln("/Y21 load 2 mul /Y22 load add 3 div");
             gen.writeln("/X22 load /Y22 load curveto");
             gen.writeln("} bd");
-            
+
             gen.writeln("/SSPD {");
             gen.writeln("dup length /d exch dict def");
             gen.writeln("{");
@@ -204,25 +247,31 @@ public final class PSProcSets {
             gen.writeDSCComment(DSCConstants.END_RESOURCE);
             gen.getResourceTracker().registerSuppliedResource(this);
         }
-        
+
+        /** {@inheritDoc} */
+        public String mapCommand(String command) {
+            String mapped = (String)STANDARD_MACROS.get(command);
+            return (mapped != null ? mapped : command);
+        }
+
     }
 
     private static class EPSProcSet extends PSProcSet {
-        
+
         public EPSProcSet() {
             super("Apache XML Graphics EPS ProcSet", 1.0f, 0);
         }
-        
+
         public void writeTo(PSGenerator gen) throws IOException {
-            gen.writeDSCComment(DSCConstants.BEGIN_RESOURCE, 
-                    new Object[] {TYPE_PROCSET, getName(), 
+            gen.writeDSCComment(DSCConstants.BEGIN_RESOURCE,
+                    new Object[] {TYPE_PROCSET, getName(),
                         Float.toString(getVersion()), Integer.toString(getRevision())});
-            gen.writeDSCComment(DSCConstants.VERSION, 
+            gen.writeDSCComment(DSCConstants.VERSION,
                     new Object[] {Float.toString(getVersion()), Integer.toString(getRevision())});
             gen.writeDSCComment(DSCConstants.COPYRIGHT, "Copyright 2002-2003 "
                     + "The Apache Software Foundation. "
                     + "License terms: http://www.apache.org/licenses/LICENSE-2.0");
-            gen.writeDSCComment(DSCConstants.TITLE, 
+            gen.writeDSCComment(DSCConstants.TITLE,
                     "EPS procedures used by the Apache XML Graphics project (Batik and FOP)");
 
             gen.writeln("/BeginEPSF { %def");
@@ -247,13 +296,13 @@ public final class PSProcSets {
             gen.writeln("countdictstack dict_count sub {end} repeat");
             gen.writeln("b4_Inc_state restore");
             gen.writeln("} bd");
-            
+
             gen.writeDSCComment(DSCConstants.END_RESOURCE);
             gen.getResourceTracker().registerSuppliedResource(this);
         }
-        
+
     }
-    
+
     /**
      * Generates a resource defining standard procset with operations used by the XML Graphics
      * project.
