@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,36 +15,38 @@
  * limitations under the License.
  */
 
-/* $Id: PSState.java 548990 2007-06-20 08:28:52Z jeremias $ */
- 
+/* $Id: PSState.java 959327 2010-06-30 14:25:03Z spepping $ */
+
 package org.apache.xmlgraphics.ps;
 
+import java.awt.Color;
+import java.awt.geom.AffineTransform;
 import java.io.IOException;
 import java.io.Serializable;
 import java.util.List;
-import java.awt.Color;
-import java.awt.geom.AffineTransform;
 
 /**
  * This class holds the current state of the PostScript interpreter.
- * 
- * @version $Id: PSState.java 548990 2007-06-20 08:28:52Z jeremias $
+ *
+ * @version $Id: PSState.java 959327 2010-06-30 14:25:03Z spepping $
  */
 public class PSState implements Serializable {
 
     /** Default for setdash */
-    public static final String DEFAULT_DASH = "[] 0"; 
+    public static final String DEFAULT_DASH = "[] 0";
     /** Default color in PostScript */
-    public static final Color DEFAULT_RGB_COLOR = Color.black; 
-    
+    public static final Color DEFAULT_RGB_COLOR = Color.black;
+
     private AffineTransform transform = new AffineTransform();
     private List transformConcatList = new java.util.ArrayList();
- 
+
     private int linecap = 0;
+    private int linejoin = 0;
+    private float miterLimit = 0;
     private double linewidth = 1.0f;
     private String dashpattern = DEFAULT_DASH;
     private Color color = DEFAULT_RGB_COLOR;
-    
+
     //Font state
     private String fontname;
     private float fontsize;
@@ -67,13 +69,15 @@ public class PSState implements Serializable {
             this.transformConcatList.addAll(org.transformConcatList);
         }
         this.linecap = org.linecap;
+        this.linejoin = org.linejoin;
+        this.miterLimit = org.miterLimit;
         this.linewidth = org.linewidth;
         this.dashpattern = org.dashpattern;
         this.color = org.color;
         this.fontname = org.fontname;
         this.fontsize = org.fontsize;
     }
- 
+
     /**
      * Returns the transform.
      * @return the current transformation matrix
@@ -94,7 +98,7 @@ public class PSState implements Serializable {
     public boolean checkTransform(AffineTransform tf) {
         return !tf.equals(this.transform);
     }
-    
+
     /**
      * Concats the given transformation matrix with the current one.
      * @param transform The new transformation matrix
@@ -117,7 +121,35 @@ public class PSState implements Serializable {
             return false;
         }
     }
-    
+
+    /**
+     * Establishes the specified line join.
+     * @param value line join (0, 1 or 2) as defined by the setlinejoin command
+     * @return true if the line join changed compared to the previous setting
+     */
+    public boolean useLineJoin(int value) {
+        if (linejoin != value) {
+            linejoin = value;
+            return true;
+        } else {
+            return false;
+        }
+    }
+
+    /**
+     * Establishes the specified miter limit.
+     * @param value the miter limit as defined by the setmiterlimit command
+     * @return true if the miter limit changed compared to the previous setting
+     */
+    public boolean useMiterLimit(float value) {
+        if (miterLimit != value) {
+            miterLimit = value;
+            return true;
+        } else {
+            return false;
+        }
+    }
+
     /**
      * Establishes the specified line width.
      * @param value line width as defined by the setlinewidth command
@@ -131,7 +163,7 @@ public class PSState implements Serializable {
             return false;
         }
     }
-    
+
     /**
      * Establishes the specified dash.
      * @param pattern dash pattern as defined by the setdash command
@@ -145,7 +177,7 @@ public class PSState implements Serializable {
             return false;
         }
     }
-    
+
     /**
      * Establishes the specified color (RGB).
      * @param value color as defined by the setrgbcolor command
@@ -159,7 +191,7 @@ public class PSState implements Serializable {
             return false;
         }
     }
-    
+
     /**
      * Establishes the specified font and size.
      * @param name name of the font for the "F" command (see FOP Std Proc Set)
@@ -178,7 +210,7 @@ public class PSState implements Serializable {
             return false;
         }
     }
-    
+
     /**
      * Reestablishes the graphics state represented by this instance by issueing the
      * necessary commands.
@@ -197,5 +229,5 @@ public class PSState implements Serializable {
             gen.useFont(fontname, fontsize);
         }
     }
-    
+
 }
