@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id$ */
+/* $Id: DefaultNestedDocumentHandler.java 734420 2009-01-14 15:38:32Z maxberger $ */
 
 package org.apache.xmlgraphics.ps.dsc;
 
@@ -27,13 +27,14 @@ import org.apache.xmlgraphics.ps.dsc.events.DSCComment;
 import org.apache.xmlgraphics.ps.dsc.events.DSCEvent;
 
 /**
- * Default implementation of the NestedDocumentHandler interface which automatically skips data
+ * {@link DSCListener} implementation which automatically skips data
  * between Begin/EndDocument and Begin/EndData.
  */
-public class DefaultNestedDocumentHandler implements DSCParserConstants, NestedDocumentHandler {
+public class DefaultNestedDocumentHandler implements DSCParserConstants,
+        NestedDocumentHandler, DSCListener {
 
     private PSGenerator gen;
-    
+
     /**
      * Creates a new instance.
      * @param gen PSGenerator to pass through the skipped content
@@ -41,37 +42,52 @@ public class DefaultNestedDocumentHandler implements DSCParserConstants, NestedD
     public DefaultNestedDocumentHandler(PSGenerator gen) {
         this.gen = gen;
     }
-    
-    /**
-     * @see org.apache.xmlgraphics.ps.dsc.NestedDocumentHandler#handle(org.apache.xmlgraphics.ps.dsc.events.DSCEvent, org.apache.xmlgraphics.ps.dsc.DSCParser)
-     */
+
+    /** {@inheritDoc} */
     public void handle(DSCEvent event, DSCParser parser) throws IOException, DSCException {
+        processEvent(event, parser);
+    }
+
+    /** {@inheritDoc} */
+    public void processEvent(DSCEvent event, DSCParser parser) throws IOException, DSCException {
         if (event.isDSCComment()) {
             DSCComment comment = event.asDSCComment();
             if (DSCConstants.BEGIN_DOCUMENT.equals(comment.getName())) {
-                comment.generate(gen);
+                if (gen != null) {
+                    comment.generate(gen);
+                }
                 parser.setCheckEOF(false);
+                parser.setListenersDisabled(true);
                 comment = parser.nextDSCComment(DSCConstants.END_DOCUMENT, gen);
                 if (comment == null) {
-                    throw new DSCException("File is not DSC-compliant: Didn't find an " 
+                    throw new DSCException("File is not DSC-compliant: Didn't find an "
                             + DSCConstants.END_DOCUMENT);
                 }
-                comment.generate(gen);
+                if (gen != null) {
+                    comment.generate(gen);
+                }
                 parser.setCheckEOF(true);
+                parser.setListenersDisabled(false);
                 parser.next();
             } else if (DSCConstants.BEGIN_DATA.equals(comment.getName())) {
-                comment.generate(gen);
+                if (gen != null) {
+                    comment.generate(gen);
+                }
                 parser.setCheckEOF(false);
+                parser.setListenersDisabled(true);
                 comment = parser.nextDSCComment(DSCConstants.END_DATA, gen);
                 if (comment == null) {
-                    throw new DSCException("File is not DSC-compliant: Didn't find an " 
+                    throw new DSCException("File is not DSC-compliant: Didn't find an "
                             + DSCConstants.END_DATA);
                 }
-                comment.generate(gen);
+                if (gen != null) {
+                    comment.generate(gen);
+                }
                 parser.setCheckEOF(true);
+                parser.setListenersDisabled(false);
                 parser.next();
-            } 
+            }
         }
     }
-    
+
 }
