@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: XMPArray.java 606564 2007-12-23 15:42:28Z jeremias $ */
+/* $Id: XMPArray.java 893118 2009-12-22 09:14:05Z jeremias $ */
 
 package org.apache.xmlgraphics.xmp;
 
@@ -27,14 +27,14 @@ import org.xml.sax.helpers.AttributesImpl;
 
 /**
  * Represents an XMP array as defined by the XMP specification.
- * @todo Property qualifiers are currently not supported, yet. 
+ * @todo Property qualifiers are currently not supported, yet.
  */
 public class XMPArray extends XMPComplexValue {
 
     private XMPArrayType type;
     private List values = new java.util.ArrayList();
     private List xmllang = new java.util.ArrayList();
-    
+
     /**
      * Main constructor
      * @param type the intended type of array
@@ -42,12 +42,12 @@ public class XMPArray extends XMPComplexValue {
     public XMPArray(XMPArrayType type) {
         this.type = type;
     }
-    
+
     /** @return the type of array */
     public XMPArrayType getType() {
         return this.type;
     }
-    
+
     /**
      * Returns the value at a given position.
      * @param idx the index of the requested value
@@ -89,7 +89,7 @@ public class XMPArray extends XMPComplexValue {
         }
         return null;
     }
-    
+
     /**
      * Returns a language-dependent values (available for alternative arrays).
      * @param lang the language ("x-default" for the default value)
@@ -124,25 +124,27 @@ public class XMPArray extends XMPComplexValue {
         }
         return v;
     }
-    
+
     /**
-     * Removes a language-dependent value
+     * Removes a language-dependent value.
      * @param lang the language ("x-default" for the default value)
+     * @return the removed value (or null if no value was set)
      */
-    public void removeLangValue(String lang) {
-        if (lang == null && "".equals(lang)) {
-            return;
+    public String removeLangValue(String lang) {
+        if (lang == null || "".equals(lang)) {
+            lang = XMPConstants.DEFAULT_LANGUAGE;
         }
         for (int i = 0, c = values.size(); i < c; i++) {
             String l = (String)xmllang.get(i);
-            if (lang.equals(l)) {
-                values.remove(i);
+            if ((XMPConstants.DEFAULT_LANGUAGE.equals(lang) && l == null) || lang.equals(l)) {
+                String value = (String)values.remove(i);
                 xmllang.remove(i);
-                return;
+                return value;
             }
         }
+        return null;
     }
-    
+
     /**
      * Adds a new value to the array
      * @param value the value
@@ -152,6 +154,22 @@ public class XMPArray extends XMPComplexValue {
         xmllang.add(null);
     }
 
+    /**
+     * Removes a value from the array. If the value doesn't exist, nothing happens.
+     * @param value the value to be removed
+     * @return true if the value was removed, false if it wasn't found
+     */
+    public boolean remove(String value) {
+        int idx = values.indexOf(value);
+        if (idx >= 0) {
+            values.remove(idx);
+            xmllang.remove(idx);
+            return true;
+        }
+        return false;
+
+    }
+
     /**
      * Adds a language-dependent value to the array. Make sure not to add the same language twice.
      * @param value the value
@@ -162,11 +180,22 @@ public class XMPArray extends XMPComplexValue {
         xmllang.add(lang);
     }
 
-    /** @return the current number of value in the array */
+    /**
+     * Returns the current number of values in the array.
+     * @return the current number of values in the array
+     */
     public int getSize() {
         return this.values.size();
     }
-    
+
+    /**
+     * Indicates whether the array is empty or not.
+     * @return true if the array is empty
+     */
+    public boolean isEmpty() {
+        return getSize() == 0;
+    }
+
     /**
      * Converts the array to an object array.
      * @return an object array of all values in the array
@@ -182,7 +211,7 @@ public class XMPArray extends XMPComplexValue {
     /** {@inheritDoc} */
     public void toSAX(ContentHandler handler) throws SAXException {
         AttributesImpl atts = new AttributesImpl();
-        handler.startElement(XMPConstants.RDF_NAMESPACE, 
+        handler.startElement(XMPConstants.RDF_NAMESPACE,
                 type.getName(), "rdf:" + type.getName(), atts);
         for (int i = 0, c = values.size(); i < c; i++) {
             String lang = (String)xmllang.get(i);
@@ -190,7 +219,7 @@ public class XMPArray extends XMPComplexValue {
             if (lang != null) {
                 atts.addAttribute(XMPConstants.XML_NS, "lang", "xml:lang", "CDATA", lang);
             }
-            handler.startElement(XMPConstants.RDF_NAMESPACE, 
+            handler.startElement(XMPConstants.RDF_NAMESPACE,
                     "li", "rdf:li", atts);
             Object v = values.get(i);
             if (v instanceof XMPComplexValue) {
@@ -200,10 +229,10 @@ public class XMPArray extends XMPComplexValue {
                 char[] chars = value.toCharArray();
                 handler.characters(chars, 0, chars.length);
             }
-            handler.endElement(XMPConstants.RDF_NAMESPACE, 
+            handler.endElement(XMPConstants.RDF_NAMESPACE,
                     "li", "rdf:li");
         }
-        handler.endElement(XMPConstants.RDF_NAMESPACE, 
+        handler.endElement(XMPConstants.RDF_NAMESPACE,
                 type.getName(), "rdf:" + type.getName());
     }
 
@@ -212,5 +241,4 @@ public class XMPArray extends XMPComplexValue {
         return "XMP array: " + type + ", " + getSize();
     }
 
-    
 }
