@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: XMPSchemaAdapter.java 652284 2008-04-30 06:17:12Z jeremias $ */
+/* $Id: XMPSchemaAdapter.java 893128 2009-12-22 10:20:22Z jeremias $ */
 
 package org.apache.xmlgraphics.xmp;
 
@@ -37,7 +37,7 @@ public class XMPSchemaAdapter {
     /** the Metadata object this schema instance operates on */
     protected Metadata meta;
     private XMPSchema schema;
-    
+
     /**
      * Main constructor.
      * @param meta the Metadata object to wrao
@@ -53,19 +53,19 @@ public class XMPSchemaAdapter {
         this.meta = meta;
         this.schema = schema;
     }
-    
+
     /** @return the XMP schema associated with this adapter */
     public XMPSchema getSchema() {
         return this.schema;
     }
-    
+
     /**
      * Returns the QName for a given property
      * @param propName the property name
      * @return the resulting QName
      */
     protected QName getQName(String propName) {
-        return new QName(getSchema().getNamespace(), propName);
+        return new QName(getSchema().getNamespace(), getSchema().getPreferredPrefix(), propName);
     }
 
     /**
@@ -75,13 +75,26 @@ public class XMPSchemaAdapter {
      * @param arrayType the type of array to operate on
      */
     private void addStringToArray(String propName, String value, XMPArrayType arrayType) {
+        if (value == null || value.length() == 0) {
+            throw new IllegalArgumentException("Value must not be empty");
+        }
+        addObjectToArray(propName, value, arrayType);
+    }
+
+    /**
+     * Adds a Object value to an array.
+     * @param propName the property name
+     * @param value the Object value
+     * @param arrayType the type of array to operate on
+     */
+    protected void addObjectToArray(String propName, Object value, XMPArrayType arrayType) {
+        if (value == null) {
+            throw new IllegalArgumentException("Value must not be null");
+        }
         QName name = getQName(propName);
         XMPProperty prop = meta.getProperty(name);
-        XMPArray array;
         if (prop == null) {
-            array = new XMPArray(arrayType);
-            array.add(value);
-            prop = new XMPProperty(name, array);
+            prop = new XMPProperty(name, value);
             meta.setProperty(prop);
         } else {
             prop.convertSimpleValueToArray(arrayType);
@@ -89,6 +102,37 @@ public class XMPSchemaAdapter {
         }
     }
 
+    /**
+     * Removes a value from an array.
+     * @param propName the name of the property
+     * @param value the value to be removed
+     * @return true if the value was removed, false if it was not found
+     */
+    protected boolean removeStringFromArray(String propName, String value) {
+        if (value == null) {
+            return false;
+        }
+        QName name = getQName(propName);
+        XMPProperty prop = meta.getProperty(name);
+        if (prop != null) {
+            if (prop.isArray()) {
+                XMPArray arr = prop.getArrayValue();
+                boolean removed = arr.remove(value);
+                if (arr.isEmpty()) {
+                    meta.removeProperty(name);
+                }
+                return removed;
+            } else {
+                Object currentValue = prop.getValue();
+                if (value.equals(currentValue)) {
+                    meta.removeProperty(name);
+                    return true;
+                }
+            }
+        }
+        return false;
+    }
+
     /**
      * Adds a String value to an ordered array.
      * @param propName the property name
@@ -122,7 +166,7 @@ public class XMPSchemaAdapter {
         df.setTimeZone(TimeZone.getTimeZone("GMT"));
         return df;
     }
-    
+
     /**
      * Formats a Date using ISO 8601 format in the given time zone.
      * @param dt the date
@@ -135,13 +179,13 @@ public class XMPSchemaAdapter {
         cal.setTime(dt);
         int offset = cal.get(Calendar.ZONE_OFFSET);
         offset += cal.get(Calendar.DST_OFFSET);
-        
+
         //DateFormat is operating on GMT so adjust for time zone offset
         Date dt1 = new Date(dt.getTime() + offset);
         StringBuffer sb = new StringBuffer(createPseudoISO8601DateFormat().format(dt1));
 
         offset /= (1000 * 60); //Convert to minutes
-        
+
         if (offset == 0) {
             sb.append('Z');
         } else {
@@ -162,10 +206,10 @@ public class XMPSchemaAdapter {
             }
             sb.append(zoneOffsetMinutes);
         }
-        
+
         return sb.toString();
     }
-    
+
     /**
      * Parses an ISO 8601 date and time value.
      * @param dt the date and time value as an ISO 8601 string
@@ -214,7 +258,7 @@ public class XMPSchemaAdapter {
         String dt = formatISO8601Date(value);
         addStringToSeq(propName, dt);
     }
-    
+
     /**
      * Set a date value.
      * @param propName the property name
@@ -238,7 +282,7 @@ public class XMPSchemaAdapter {
             return parseISO8601Date(dt);
         }
     }
-    
+
     /**
      * Sets a language-dependent value.
      * @param propName the property name
@@ -253,14 +297,22 @@ public class XMPSchemaAdapter {
         XMPProperty prop = meta.getProperty(name);
         XMPArray array;
         if (prop == null) {
-            array = new XMPArray(XMPArrayType.ALT);
-            array.add(value, lang);
-            prop = new XMPProperty(name, array);
-            meta.setProperty(prop);
+            if (value != null && value.length() > 0) {
+                prop = new XMPProperty(name, value);
+                prop.setXMLLang(lang);
+                meta.setProperty(prop);
+            }
         } else {
             prop.convertSimpleValueToArray(XMPArrayType.ALT);
-            removeLangAlt(lang, propName);
-            prop.getArrayValue().add(value, lang);
+            array = prop.getArrayValue();
+            array.removeLangValue(lang);
+            if (value != null && value.length() > 0) {
+                array.add(value, lang);
+            } else {
+                if (array.isEmpty()) {
+                    meta.removeProperty(name);
+                }
+            }
         }
     }
 
@@ -272,11 +324,17 @@ public class XMPSchemaAdapter {
     protected void setValue(String propName, String value) {
         QName name = getQName(propName);
         XMPProperty prop = meta.getProperty(name);
-        if (prop == null) {
-            prop = new XMPProperty(name, value);
-            meta.setProperty(prop);
+        if (value != null && value.length() > 0) {
+            if (prop != null) {
+                prop.setValue(value);
+            } else {
+                prop = new XMPProperty(name, value);
+                meta.setProperty(prop);
+            }
         } else {
-            prop.setValue(value);
+            if (prop != null) {
+                meta.removeProperty(name);
+            }
         }
     }
 
@@ -299,23 +357,31 @@ public class XMPSchemaAdapter {
      * Removes a language-dependent value from an alternative array.
      * @param lang the language ("x-default" for the default language)
      * @param propName the property name
+     * @return the removed value
      */
-    protected void removeLangAlt(String lang, String propName) {
-        XMPProperty prop = meta.getProperty(getQName(propName));
+    protected String removeLangAlt(String lang, String propName) {
+        QName name = getQName(propName);
+        XMPProperty prop = meta.getProperty(name);
         XMPArray array;
         if (prop != null && lang != null) {
             array = prop.getArrayValue();
             if (array != null) {
-                array.removeLangValue(lang);
-            } else {
-                if (lang.equals(prop.getXMLLang())) {
-                    prop.setValue(null);
-                    prop.setXMLLang(null);
+                String removed = array.removeLangValue(lang);
+                if (array.isEmpty()) {
+                    meta.removeProperty(name);
                 }
+                return removed;
+            } else {
+                String removed = prop.getValue().toString();
+                if (lang.equals(prop.getXMLLang())) {
+                    meta.removeProperty(name);
+                }
+                return removed;
             }
         }
+        return null;
     }
-    
+
     /**
      * Returns a language-dependent value. If the value in the requested language is not available
      * the value for the default language is returned.
@@ -338,6 +404,60 @@ public class XMPSchemaAdapter {
         }
     }
 
+    /**
+     * Finds a structure that matches a given qualifier.
+     * @param propName the property name
+     * @param qualifier the qualifier
+     * @param qualifierValue the qualifier value
+     * @return the structure if a match was found (or null if no match was found)
+     */
+    protected PropertyAccess findQualifiedStructure(String propName,
+            QName qualifier, String qualifierValue) {
+        XMPProperty prop = meta.getProperty(getQName(propName));
+        XMPArray array;
+        if (prop != null) {
+            array = prop.getArrayValue();
+            if (array != null) {
+                for (int i = 0, c = array.getSize(); i < c; i++) {
+                    Object value = array.getValue(i);
+                    if (value instanceof PropertyAccess) {
+                        PropertyAccess pa = (PropertyAccess)value;
+                        XMPProperty q = pa.getProperty(qualifier);
+                        if (q != null && q.getValue().equals(qualifierValue)) {
+                            return pa;
+                        }
+                    }
+                }
+            } else if (prop.getStructureValue() != null) {
+                PropertyAccess pa = prop.getStructureValue();
+                XMPProperty q = pa.getProperty(qualifier);
+                if (q != null && q.getValue().equals(qualifierValue)) {
+                    return pa;
+                }
+            }
+        }
+        return null;
+    }
+
+    /**
+     * Finds a value that matches a given qualifier.
+     * @param propName the property name
+     * @param qualifier the qualifier
+     * @param qualifierValue the qualifier value
+     * @return the value if a match was found (or null if no match was found)
+     */
+    protected Object findQualifiedValue(String propName,
+            QName qualifier, String qualifierValue) {
+        PropertyAccess pa = findQualifiedStructure(propName, qualifier, qualifierValue);
+        if (pa != null) {
+            XMPProperty rdfValue = pa.getValueProperty();
+            if (rdfValue != null) {
+                return rdfValue.getValue();
+            }
+        }
+        return null;
+    }
+
     /**
      * Returns an object array representation of the property's values.
      * @param propName the property name
@@ -357,7 +477,7 @@ public class XMPSchemaAdapter {
     }
 
     /**
-     * Returns a String array representation of the property's values. Complex values to converted
+     * Returns a String array representation of the property's values. Complex values are converted
      * to Strings using the toString() method.
      * @param propName the property name
      * @return the String array or null if the property isn't set
@@ -369,10 +489,38 @@ public class XMPSchemaAdapter {
         }
         String[] res = new String[arr.length];
         for (int i = 0, c = res.length; i < c; i++) {
-            res[i] = arr[i].toString();
+            Object o = arr[i];
+            if (o instanceof PropertyAccess) {
+                XMPProperty prop = ((PropertyAccess)o).getValueProperty();
+                res[i] = prop.getValue().toString();
+            } else {
+                res[i] = o.toString();
+            }
         }
         return res;
     }
 
-    
+    /**
+     * Returns a Date array representation of the property's values.
+     * @param propName the property name
+     * @return the Date array or null if the property isn't set
+     */
+    protected Date[] getDateArray(String propName) {
+        Object[] arr = getObjectArray(propName);
+        if (arr == null) {
+            return null;
+        }
+        Date[] res = new Date[arr.length];
+        for (int i = 0, c = res.length; i < c; i++) {
+            Object obj = arr[i];
+            if (obj instanceof Date) {
+                res[i] = (Date)((Date)obj).clone();
+            } else {
+                res[i] = parseISO8601Date(obj.toString());
+            }
+        }
+        return res;
+    }
+
+
 }
